@@ -8,8 +8,9 @@ use WWW::Crawler::Mojo::Queue::Memory;
 use WWW::Crawler::Mojo::UserAgent;
 use WWW::Crawler::Mojo::ScraperUtil qw{
             collect_urls_css html_handlers resolve_href decoded_body};
+use Mojo::Util 'deprecated';
 use Mojo::Message::Request;
-our $VERSION = '0.12';
+our $VERSION = '0.13';
 
 has clock_speed => 0.25;
 has max_conn => 1;
@@ -45,7 +46,17 @@ sub init {
     $self->ua->max_redirects(5);
     
     Mojo::IOLoop->recurring($self->clock_speed => sub {
-        $self->process_job(@_);
+        my $queue = $self->queue;
+        
+        if (!$queue->length) {
+            $self->emit('empty') if (!$self->ua->active_conn);
+            return;
+        }
+        if ($self->ua->active_conn >= $self->max_conn ||
+            $self->ua->active_host($queue->next->url) >= $self->max_conn_per_host) {
+            return;
+        }
+        $self->process_job($queue->dequeue);
     });
     
     if ($self->shuffle) {
@@ -56,19 +67,7 @@ sub init {
 }
 
 sub process_job {
-    my $self = shift;
-    my $queue = $self->queue;
-    
-    if (!$queue->length) {
-        $self->emit('empty') if (!$self->ua->active_conn);
-        return;
-    }
-    if ($self->ua->active_conn >= $self->max_conn ||
-        $self->ua->active_host($queue->next->url) >= $self->max_conn_per_host) {
-        return;
-    }
-    
-    my $job = $queue->dequeue;
+    my ($self, $job) = @_;
     my $url = $job->url;
     my $ua = $self->ua;
     my $tx = $ua->build_tx($job->method || 'get' => $url => $job->tx_params);
@@ -87,7 +86,7 @@ sub process_job {
         }
         
         $self->emit('res',
-                        sub { $self->scrape($res, $job, $_[0]) }, $job, $res);
+                sub { $self->scrape($res, $job, $_[0]) }, $job, $res);
         
         $job->close;
     });
@@ -106,7 +105,11 @@ EOF
 }
 
 sub scrape {
-    my ($self, $res, $job, $cb) = @_;
+    die 'scrape method with 4 argments has been removed'
+                                    if (scalar @_ == 5 || ref $_[3] eq 'CODE');
+    
+    my ($self, $res, $job, $contexts) = @_;
+    my @ret;
     
     return unless $res->headers->content_length && $res->body;
     
@@ -118,12 +121,12 @@ sub scrape {
             $base = resolve_href($base, $base_tag->attr('href'));
         }
         my $dom = Mojo::DOM->new(decoded_body($res));
-        state $handlers = html_handlers();
+        my $handlers = html_handlers($contexts);
         for my $selector (sort keys %{$handlers}) {
             $dom->find($selector)->each(sub {
                 my $dom = shift;
                 for ($handlers->{$selector}->($dom)) {
-                    $self->_delegate_enqueue($_, $dom, $job, $base, $cb);
+                    push(@ret, $self->_make_child($_, $dom, $job, $base));
                 }
             });
         }
@@ -131,13 +134,15 @@ sub scrape {
     
     if ($type && $type =~ qr{text/css}) {
         for (collect_urls_css(decoded_body($res))) {
-            $self->_delegate_enqueue($_, $job->url, $job, $base, $cb);
+            push(@ret, $self->_make_child($_, $job->url, $job, $base));
         }
     }
-};
+    
+    return @ret;
+}
 
-sub _delegate_enqueue {
-    my ($self, $url, $context, $job, $base, $cb) = @_;
+sub _make_child {
+    my ($self, $url, $context, $job, $base) = @_;
     my $method, my $params;
     
     return unless $url;
@@ -147,7 +152,8 @@ sub _delegate_enqueue {
     
     return unless ($resolved->scheme =~ qr{http|https|ftp|ws|wss});
     
-    my $child = $job->child(url => $resolved, literal_uri => $url);
+    my $child =
+        $job->child(url => $resolved, literal_uri => $url, context => $context);
     
     $child->method($method) if $method;
     
@@ -159,8 +165,7 @@ sub _delegate_enqueue {
         }
     }
     
-    $cb ||= sub { $_[1]->() };
-    $cb->($self, sub { $self->enqueue($_[0] || $child) }, $child, $context);
+    return $child;
 }
 
 sub enqueue {
@@ -198,11 +203,7 @@ WWW::Crawler::Mojo - A web crawling framework for Perl
     $bot->on(res => sub {
         my ($bot, $scrape, $job, $res) = @_;
         
-        $scrape->(sub {
-            my ($bot, $enqueue, $job2, $context) = @_;
-            
-            $enqueue->();
-        });
+        $bot->enqueue($_) for $scrape->('#context');
     });
     
     $bot->enqueue('http://example.com/');
@@ -213,8 +214,8 @@ WWW::Crawler::Mojo - A web crawling framework for Perl
 L<WWW::Crawler::Mojo> is a web crawling framework for those who familier with
 L<Mojo>::* APIs.
 
-Note that the module is aimed at trivial use cases of crawling within a
-moderate range of web pages so DO NOT use it for persistent crawler jobs.
+Althogh the module is only well tested for "focused crawl" at this point,
+you can also use it for endless crawling by taking special care of memory usage.
 
 =head1 ATTRIBUTES
 
@@ -223,21 +224,21 @@ implements the following new ones.
 
 =head2 clock_speed
 
-A number of millisecond for main event loop interval. Defaults to 0.25.
+A number of main event loop interval in milliseconds. Defaults to 0.25.
 
     $bot->clock_speed(2);
     my $clock = $bot->clock_speed; # 2
 
 =head2 max_conn
 
-A number of max connections.
+An amount of max connections.
 
     $bot->max_conn(5);
     say $bot->max_conn; # 5
 
 =head2 max_conn_per_host
 
-A number of max connections per host.
+An amount of max connections per host.
 
     $bot->max_conn_per_host(5);
     say $bot->max_conn_per_host; # 5
@@ -259,10 +260,10 @@ for disabling/enabling the feature. Defaults to undef, meaning disable.
 
 =head2 ua
 
-A L<Mojo::UserAgent> instance.
+A L<WWW::Crawler::Mojo::UserAgent> instance.
 
     my $ua = $bot->ua;
-    $bot->ua(Mojo::UserAgent->new);
+    $bot->ua(WWW::Crawler::Mojo::UserAgent->new);
 
 =head2 ua_name
 
@@ -283,9 +284,7 @@ Emitted when crawler got response from server. The callback takes 4 arguments.
     $bot->on(res => sub {
         my ($bot, $scrape, $job, $res) = @_;
         if (...) {
-            $scrape->(sub {
-                # called when URL found
-            });
+            $bot->enqueue($_) for $scrape->();
         } else {
             # DO NOTHING
         }
@@ -297,35 +296,17 @@ L<WWW::Crawler::Mojo> instance.
 
 =head3 $scrape
 
-Scraper code reference for current document. The code takes a callback for
-argument in case a URL found.
+Scraper code reference for current document. The code takes optional argument
+CSS selector for context and returns new jobs.
 
-    $scrape(sub {
-        my ($bot, $enqueue, $job, $context) = @_;
-        ...
-    });
+    for my $job ($scrape->($context)) {
+        $bot->enqueue($job)
+    }
 
-=over
+Optionally you can specify a scraping target container in CSS selector.
 
-=item $bot
-
-L<WWW::Crawler::Mojo> instance.
-
-=item $enqueue
-
-Enqueue code reference for current URL. This is a shorthand of..
-
-    $bot->enqueue($job)
-
-=item $job
-
-L<WWW::Crawler::Mojo::Job> instance.
-
-=item $context
-
-Either L<Mojo::DOM> or L<Mojo::URL> instance.
-
-=back
+    @jobs = $scrape->('#container');
+    @jobs = $scrape->(['#container1', '#container2']);
 
 =head3 $job
 
@@ -337,7 +318,7 @@ L<Mojo::Message::Response> instance.
 
 =head2 empty
 
-Emitted when queue length got zero. The length is checked every 5 seconds.
+Emitted when queue length gets zero.
 
     $bot->on(empty => sub {
         my ($bot) = @_;
@@ -400,10 +381,13 @@ Displays starting messages to STDOUT
 
 =head2 scrape
 
-Parses and discovers links in a web page. Each links are appended to FIFO array.
-This performs scraping.
+Parses and discovers links in a web page and CSS. This performs scraping.
+With the optional 4th argument, you can specify a CSS selector to container
+you would collect URLs within.
 
-    $bot->scrape($res, $job, $cb);
+    $bot->scrape($res, $job, );
+    $bot->scrape($res, $job, $selector);
+    $bot->scrape($res, $job, [$selector1, $selector2]);
 
 =head2 enqueue
 
@@ -449,7 +433,7 @@ L<https://github.com/jamadam/WWW-Flatten>
 
 =head1 AUTHOR
 
-Sugama Keita, E<lt>sugama@jamadam.comE<gt>
+Keita Sugama, E<lt>sugama@jamadam.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
