@@ -84,10 +84,10 @@ my $handlers = {
     'meta[content]' => sub {
         return $1 if ($_[0] =~ qr{http\-equiv="?Refresh"?}i &&
                                 (($_[0]->{content} || '') =~ qr{URL=(.+)}i)[0]);
+        return;
     },
     'style' => sub {
-        my $dom = shift;
-        return collect_urls_css($dom->content);
+        collect_urls_css(shift->content);
     },
     '[style]' => sub {
         collect_urls_css(shift->{style});
@@ -98,14 +98,19 @@ my $handlers = {
 };
 
 sub html_handlers {
+    my $contexts = ref $_[0] ? $_[0] : [$_[0]];
+    my $ret;
     for (keys %$handlers) {
         my $cb = $handlers->{$_};
-        $handlers->{$_} = sub {
-            return if ($_[0]->xml && _wrong_dom_detection($_[0]));
-            return $cb->($_[0]);
+        for my $cont (@$contexts) {
+            my $key = ($cont ? $cont. ' ' : ''). $_;
+            $ret->{$key} = sub {
+                return if ($_[0]->xml && _wrong_dom_detection($_[0]));
+                return $cb->($_[0]);
+            }
         }
     }
-    return $handlers;
+    return $ret;
 }
 
 sub resolve_href {
@@ -183,9 +188,11 @@ Generates L<Encode> instance for given name. Defaults to L<Encode::utf8>.
 
 =head2 html_handlers
 
-HTML element handler presets on scraping.
+HTML element handler presets on scraping. Optional argument narrows the preset
+selector into certain containers.
 
-    my $handlers = html_handlers();
+    my $handlers = html_handlers(['#header', '#footer li']);
+    
     $handlers->{img} = sub {
         my $dom = shift;
         return $dom->{src};
@@ -212,11 +219,11 @@ Guesses encoding of HTML or CSS with given L<Mojo::Message::Response> instance.
 
 =head1 AUTHOR
 
-Sugama Keita, E<lt>sugama@jamadam.comE<gt>
+Keita Sugama, E<lt>sugama@jamadam.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) Sugama Keita.
+Copyright (C) Keita Sugama.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
