@@ -6,8 +6,8 @@ use Mojo::URL;
 use 5.010;
 
 has active_conn => 0;
-has active_conns_per_host => sub { {} };
-has credentials => sub {{}};
+has active_conn_per_host => sub { {} };
+has '_creds';
 has keep_credentials => 1;
 
 sub new {
@@ -15,14 +15,15 @@ sub new {
     my $self = $class->SUPER::new(@_);
     
     if ($self->keep_credentials) {
+        $self->_creds({});
         $self->on(start => sub {
             my ($self, $tx) = @_;
             my $url = $tx->req->url;
             my $host_key = _host_key($url) or return;
             if ($url->userinfo) {
-                $self->credentials->{$host_key} = $url->userinfo;
+                $self->{_creds}->{$host_key} = $url->userinfo;
             } else {
-                $url->userinfo($self->credentials->{$host_key});
+                $url->userinfo($self->{_creds}->{$host_key});
             }
         });
     }
@@ -40,7 +41,7 @@ sub new {
 sub active_host {
     my ($self, $url, $inc) = @_;
     my $key = _host_key($url);
-    my $hosts = $self->active_conns_per_host;
+    my $hosts = $self->active_conn_per_host;
     if ($inc) {
         $self->{active_conn} += $inc;
         $hosts->{$key} += $inc;
@@ -49,6 +50,13 @@ sub active_host {
     return $hosts->{$key} || 0;
 }
 
+sub credentials {
+    my ($self, %credentials) = @_;
+    while (my ($url, $cred) = each(%credentials)) {
+        $self->{_creds}->{_host_key($url)} = $cred;
+    }
+}
+
 sub _host_key {
     state $well_known_ports = {http => 80, https => 443};
     my $url = shift;
@@ -64,20 +72,31 @@ sub _host_key {
 
 =head1 NAME
 
-WWW::Crawler::Mojo::UserAgent - Mojo::UserAgent sub class for userinfo storage
+WWW::Crawler::Mojo::UserAgent - Crawler specific featured user agent
 
 =head1 SYNOPSIS
 
     my $ua = WWW::Crawler::Mojo::UserAgent->new;
     $ua->keep_credentials(1);
-    $ua->credentials->{'http://example.com:80'} = 'jamadam:password';
+    $ua->credentials(
+        'http://example.com:8080' => 'jamadam:password1',
+        'http://example2.com:8080' => 'jamadam:password2',
+    );
     my $tx = $ua->get('http://example.com/');
     say $tx->req->url # http://jamadam:passowrd@example.com/
+    
+    if ($ua->active_conn < $max_conn) {
+        $ua->get(...);
+    }
+    
+    if ($ua->active_host($url) < $max_conn_per_host) {
+        $ua->get(...);
+    }
 
 =head1 DESCRIPTION
 
-This class inherits L<Mojo::UserAgent> and override start method for storing user
-info
+This class inherits L<Mojo::UserAgent> and adds credential storage and
+active connection counter.
 
 =head1 ATTRIBUTES
 
@@ -90,25 +109,19 @@ A number of current connections.
     $bot->active_conn($bot->active_conn + 1);
     say $bot->active_conn;
 
-=head2 active_conns_per_host
+=head2 active_conn_per_host
 
 A number of current connections per host.
 
-    $bot->active_conns_per_host($bot->active_conns_per_host + 1);
-    say $bot->active_conns_per_host;
+    $bot->active_conn_per_host($bot->active_conn_per_host + 1);
+    say $bot->active_conn_per_host;
 
 =head2 keep_credentials
 
-Set true to set the feature on, defaults to 1.
+Set true to activate the feature. Defaults to 1.
 
     $ua->keep_credentials(1);
 
-=head2 credentials
-
-Storage for credentials.
-
-    $ua->credentials->{'http://example.com:80'} = 'jamadam:password';
-
 =head1 METHODS
 
 WWW::Crawler::Mojo::UserAgent inherits all methods from L<Mojo::UserAgent>.
@@ -121,6 +134,15 @@ Maintenance the numbers of active connections.
     $ua->active_host($url, -1);
     my $amount = $ua->active_host($url);
 
+=head2 credentials
+
+Stores credentials.
+
+    $ua->credentials(
+        'http://example.com:8080' => 'jamadam:password1',
+        'http://example2.com:8080' => 'jamadam:password2',
+    );
+
 =head2 new
 
 Constructer.
@@ -129,11 +151,11 @@ Constructer.
 
 =head1 AUTHOR
 
-Sugama Keita, E<lt>sugama@jamadam.comE<gt>
+Keita Sugama, E<lt>sugama@jamadam.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) Sugama Keita.
+Copyright (C) Keita Sugama.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
